import streamlit as st
import pandas as pd
import datetime
import humanize

from team_assignment import TeamAssignment
from ortools.sat.python import cp_model

class SolutionCallback(cp_model.CpSolverSolutionCallback):
    def __init__(self):
        cp_model.CpSolverSolutionCallback.__init__(self)
        self.start_time = datetime.datetime.now()

    def on_solution_callback(self):
        objective_value = self.ObjectiveValue()
        num_conflicts = self.NumConflicts()
        cur_time = datetime.datetime.now()
        time_diff = cur_time - self.start_time
        time_diff_human = humanize.naturaldelta(time_diff)
        print(
            f"Elapsed time: {time_diff}, Number of conflicts: " +
            f"{num_conflicts}, Objective value: {objective_value}"
        )
        if time_diff.seconds > 10:
            self.StopSearch()

def roster_upload_callback():
    roster_upload = st.session_state["roster_upload"]
    if roster_upload.type == "text/csv":
         roster = (pd.read_csv(roster_upload)
                   .pipe(create_list_columns))
    elif roster_upload.type == "application/json":
        roster = pd.read_json(roster_upload)
    st.session_state["roster"] = roster

def create_list_columns(df):
    new_df = df.copy()
    for list_col in (col for col in df.columns
                     if col.endswith("_list")):
        new_df[list_col] = new_df[list_col].map(lambda str_val: str_val.split(";"))
    return new_df
    
def constraints_upload_callback():
    constraints_upload = st.session_state["constraints_upload"]
    if constraints_upload.type == "text/csv":
         constraints = pd.read_csv(constraints_upload)
    elif constraints_upload.type == "application/json":
        constraints = pd.read_json(constraints_upload)
    st.session_state["constraints"] = constraints

def solve_assignment():
    st.session_state["team_assignment"].solve(SolutionCallback())

def generate_teams_callback():
    team_assignment = TeamAssignment(
        st.session_state["roster"],
        st.session_state["constraints"],
        st.session_state["target_team_size"])
    st.session_state["team_assignment"] = team_assignment
    callback = SolutionCallback()
    with st.spinner("Generating teams..."):
        team_assignment.solve(solution_callback=callback)
        if team_assignment.solution_found:
            st.session_state["solution_found"] = True
            roster_teams = (team_assignment
                            .participants
                            .sort_values("team_num"))
            st.session_state["roster"] = roster_teams
            roster_csv = roster_teams.to_csv(index=False).encode("utf-8")
            st.session_state["roster_csv"] = roster_csv

st.title("Team Formation")

st.subheader("Roster")

if "roster" in st.session_state:
    roster = st.session_state["roster"]
    st.dataframe(roster, hide_index=True)
else:
    st.write("*Please upload participant roster*")

if "solution_found" in st.session_state:
    st.download_button(label="Download roster teams",
                       data=st.session_state["roster_csv"],
                       file_name="roster_team_assignments.csv",
                       mime="text/csv")

st.subheader("Constraints")
    
if "constraints" in st.session_state:
    constraints = st.session_state["constraints"]
    st.dataframe(constraints, hide_index=True)
else:
    st.write("*Please upload team assignment constraints*")

if (("target_team_size" in st.session_state) and
    ("over_under_size" in st.session_state) and
    ("roster" in st.session_state) and
    ("constraints" in st.session_state)):
    st.button("Generate teams",
              on_click=generate_teams_callback)
    
st.subheader("Data Upload and Setup")

st.number_input("Target team size",
                min_value=2,
                key="target_team_size")
                
st.selectbox("Team sizes over/under",
             options=["Over", "Under"],
             key="over_under_size")
                
st.file_uploader("Participant roster",
                 type=["csv", "json"],
                 key="roster_upload",
                 on_change=roster_upload_callback)

st.file_uploader("Assignment constraints",
                 type=["csv", "json"],
                 key="constraints_upload",
                 on_change=constraints_upload_callback)




import os
import pandas as pd

from team_formation.team_assignment import TeamAssignment

TEST_DIR = os.path.dirname(os.path.abspath(__file__))
BIG_TEST_ROSTER = os.path.join(TEST_DIR, "data", "big_test_roster.json")

def test_big():
    participants_all = pd.read_json(BIG_TEST_ROSTER)
    participants = participants_all[214:].copy()
    constraints = pd.DataFrame(
        columns=["attribute", "type", "weight"],
        data=[["gender", "diversify", 1],
              ["working_time", "cluster", 1]]
    )
    target_team_size = 10
    ta = TeamAssignment(
        participants,
        constraints,
        target_team_size)

    # Check matching number of participants
    assert(ta.num_participants == len(participants))
    # No solution should be found yet
    assert(not ta.solution_found)
    
    ta.solve()

    # Solution should be found
    assert(ta.solution_found)
    # Resulting teams should all be of size 10
    assert(all(ta.participants["team_num"].value_counts() == 10))

    print(ta.participants.sort_values("team_num").to_string())
    team_eval = ta.evaluate_teams()

    # There should be only one miss
    assert(max(team_eval["missed"] > 0) == 1)
    print(team_eval.to_string())

    



from setuptools import setup

with open("team_formation/_version.txt") as f:
    version = f.read().strip()

setup(
    name="team-formation",
    version=version,
    description=(
        "A tool to form teams from a larger group based "
        "on clustering and diversity constraints"
    ),
    url="https://github.com/harvard-hbs/team-formation/",
    author="Brent Benson",
    license="MIT",
    install_requires=[
        "pandas>=2.0",
        "ortools>=9.2",
        "streamlit>=1.30",
        "humanize>=4.9",
    ],
    package_data={
        "team_formation": ["_version.txt"],
    }
)



import pandas as pd

from team_formation.team_assignment import TeamAssignment

def test_small():
    participants = pd.DataFrame(
        columns=["id", "gender", "job_function", "working_time"],
        data=[[8, "Male", "Manager", ["00-05", "20-24"]],
              [9, "Male", "Executive", ["10-15", "15-20"]],
              [10, "Female", "Executive", ["15-20"]],
              [16, "Male", "Manager", ["15-20", "20-24"]],
              [18, "Female", "Contributor", ["05-10", "10-15"]],
              [20, "Female", "Manager", ["15-20", "20-24"]],
              [21, "Male", "Executive", ["15-20"]],
              [29, "Male", "Contributor", ["05-10", "10-15"]],
              [31, "Female", "Contributor", ["05-10"]]]
    )
    target_team_size = 3
    constraints = pd.DataFrame(
        columns=["attribute", "type", "weight"],
        data=[["gender", "diversify", 1],
              ["job_function", "cluster", 1],
              ["working_time", "cluster", 1]]
    )
    print(participants)
    ta = TeamAssignment(
        participants,
        constraints,
        target_team_size)

    # Check matching number of participants
    assert(ta.num_participants == len(participants))
    # No solution should be found yet
    assert(not ta.solution_found)
    
    ta.solve()

    # Solution should be found
    assert(ta.solution_found)
    # Resulting teams should all be of size 3
    assert(all(ta.participants["team_num"].value_counts() == 3))
    
    print(ta.participants.sort_values("team_num"))
    team_eval = ta.evaluate_teams()

    # There should be only one miss
    assert(sum(team_eval["missed"] > 0) == 1)
    print(team_eval)
